package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentMiniGameCallbackMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentCallbackDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameCallback;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameCallbackService;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.pojo.vo.AgentRpcVO;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDate;

@Slf4j
@Service
public class GameTencentMiniGameCallbackServiceImpl extends ServiceImpl<GameTencentMiniGameCallbackMapper, GameTencentMiniGameCallback>
        implements IGameTencentMiniGameCallbackService {

    @Value("${spring.kafka.tencentCallbackTopic}")
    private String tencentCallbackTopic;

    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;

    @Autowired
    private KafkaProducer<String, String> kafkaProducer;

    @Override
    public boolean callback(GameTencentCallbackDTO dto) {
        AgentRpcVO agent = agentRpc.getByTencentAccountId(dto.getAccountId()).getData();
        if (agent == null) {
            log.error("腾讯检测链接数据找不到渠道：{}", JsonUtil.toString(dto));
        }
        GameTencentMiniGameCallback callback = BeanUtil.copy(dto, GameTencentMiniGameCallback.class);
        if (agent != null) {
            callback.setAgentKey(agent.getAgentKey());
            callback.setGameId(agent.getGameId());
        } else {
            callback.setAgentKey("-");
            callback.setGameId(-1L);
        }
        callback.setDay(dto.getClickTime() == null || dto.getClickTime() < 1000 ? LocalDate.now() : DateUtil.secondToLocalDate(dto.getClickTime()));
        save(callback);
        try {
            kafkaProducer.send(new ProducerRecord<>(tencentCallbackTopic, dto.getAccountId().toString(), JsonUtil.toString(callback)));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return true;
    }
}
